const Main = imports.ui.main;
const Lang = imports.lang;
const Mainloop = imports.mainloop;
const Panel = imports.ui.panel;
const St = imports.gi.St;
const Clutter = imports.gi.Clutter;
const Cairo = imports.cairo;
const Gio = imports.gi.Gio;

const UPDATE_INTERVAL = 100;
const EYE_WIDTH = 1.5;
const IRIS_WIDTH = 1.5;
const PUPIL_WIDTH = 0.0;

const MARGIN = 1;

var _update_handler = null;
var _repaint_handler = null;

var eye_area, button;

function on_timeout() {
	eye_area.queue_repaint();
	return true;
}

function get_pos() {
	let area_x = 0;
	let area_y = 0;
		
	let [tx, ty] = Main.panel.actor.get_position();
	area_x += tx;
	area_y += ty;
	
	let [tx, ty] = Main.panel._rightBox.get_position();
	area_x += tx;
	area_y += ty;
	
	let [tx, ty] = this.button.get_position();
	area_x += tx;
	area_y += ty;
	
	let [tx, ty] = this.eye_area.get_position();
	area_x += tx;
	area_y += ty;
	
	return [area_x, area_y];
}

function paint_eye(area) {
	let area_width = area.get_width();
	let area_height = area.get_height();
	
	let eye_rad = area_height / 2.3;
	let iris_rad = eye_rad*0.7;
	let pupil_rad = iris_rad*0.3;
	
	let [area_x, area_y] = get_pos();
	area_x += area_width/2;
	area_y += area_height/2;
	
	let [mouse_x, mouse_y, mask] = global.get_pointer();
	mouse_x -= area_x;
	mouse_y -= area_y;
	
	let mouse_theta = -Math.atan2(mouse_x, mouse_y) +Math.PI/2;
	let mouse_rad = Math.sqrt(mouse_x*mouse_x + mouse_y*mouse_y);
	let max_rad = eye_rad*Math.cos( Math.asin((iris_rad+IRIS_WIDTH/2)/eye_rad) );
	if(mouse_rad > max_rad)
		mouse_rad = max_rad;
	
	let cr = area.get_context();
	let theme_node = eye_area.get_theme_node();
	Clutter.cairo_set_source_color(cr, theme_node.get_foreground_color());
	
	cr.translate(area_width / 2.0, area_height / 2.0);
	
	cr.setLineWidth(EYE_WIDTH);
	cr.arc(0, 0, eye_rad, 0, 2 * Math.PI);
	cr.stroke();		
	
	cr.rotate(mouse_theta);
	cr.translate(mouse_rad,0);
	
	cr.scale(Math.cos(Math.asin(mouse_rad/(eye_rad-EYE_WIDTH))),1);
	cr.setLineWidth(IRIS_WIDTH);
	cr.arc(0, 0, iris_rad - IRIS_WIDTH, 0, 2 * Math.PI);
	cr.stroke();
	
	cr.setLineWidth(PUPIL_WIDTH);
	cr.arc(IRIS_WIDTH, 0, pupil_rad - PUPIL_WIDTH, 0, 2 * Math.PI);
	cr.fill();
	
	cr.save();
	cr.restore();
}

function init() {
}

function enable() {
	eye_area = new St.DrawingArea();

	eye_area.set_width(Panel.PANEL_ICON_SIZE - 2 * MARGIN);
	eye_area.set_height(Panel.PANEL_ICON_SIZE - 2 * MARGIN);
	
	_repaint_handler = eye_area.connect("repaint", Lang.bind(this, paint_eye));
	_update_handler = Mainloop.timeout_add(UPDATE_INTERVAL, Lang.bind(this, on_timeout));
	
	button = new St.Bin({ style_class: 'panel-button',
						  reactive: true,
						  can_focus: true,
						  x_fill: true,
						  y_fill: false,
						  track_hover: true });
	button.set_width(Panel.PANEL_ICON_SIZE + 8);
	
	button.add_actor(eye_area);
	Main.panel._rightBox.insert_actor(button, 0);
	
	eye_area.queue_repaint();
}

function disable() {
	if(_repaint_handler)
		eye_area.disconnect(_repaint_handler);
	if(_update_handler)
		Mainloop.source_remove(_update_handler);

	this.button.remove_actor(eye_area);
	Main.panel._rightBox.remove_actor(button);
}
